@@ -271,7 +271,12 @@ function virtualenvwrapper_verify_active_environment {
 
 # Help text for mkvirtualenv
 function mkvirtualenv_help {
-    echo "Usage: mkvirtualenv [-r requirements_file] [virtualenv options] env_name"
+    echo "Usage: mkvirtualenv [-i package] [-r requirements_file] [virtualenv options] env_name"
+    echo
+    echo " -i package"
+    echo
+    echo "    Install a package after the environment is created."
+    echo "    This option may be repeated."
     echo
     echo " -r requirements_file"
     echo
@@ -296,6 +301,7 @@ function mkvirtualenv {
     typeset a
     typeset envname
     typeset requirements
+    typeset packages
 
     in_args=( "$@" )
 
@@ -315,6 +321,9 @@ function mkvirtualenv {
             -h)
                 mkvirtualenv_help;
                 return;;
+            -i)
+                i=$(( $i + 1 ));
+                packages="$packages ${in_args[$i]}";;
             -r)
                 i=$(( $i + 1 ));
                 requirements="${in_args[$i]}";;
@@ -356,6 +365,11 @@ function mkvirtualenv {
         pip install -r "$requirements"
     fi
 
+    for a in $packages
+    do
+        pip install $a
+    done
+
     virtualenvwrapper_run_hook "post_mkvirtualenv"
 }
 
@@ -579,7 +593,6 @@ function virtualenvwrapper_get_site_packages_dir {
 # site-packages directory; if this file does not exist, it will be
 # created first.
 function add2virtualenv {
-
     virtualenvwrapper_verify_workon_home || return 1
     virtualenvwrapper_verify_active_environment || return 1
     
@@ -591,7 +604,9 @@ function add2virtualenv {
         return 1
     fi
     
-    path_file="$site_packages/virtualenv_path_extensions.pth"
+    # Prefix with _ to ensure we are loaded as early as possible,
+    # and at least before easy_install.pth.
+    path_file="$site_packages/_virtualenv_path_extensions.pth"
 
     if [ "$*" = "" ]
     then
@@ -600,12 +615,24 @@ function add2virtualenv {
         then
             echo
             echo "Existing paths:"
-            cat "$path_file"
+            cat "$path_file" | grep -v "^import"
         fi
         return 1
     fi
 
-    touch "$path_file"
+    remove=0
+    if [ "$1" = "-d" ]
+    then
+        remove=1
+        shift
+    fi
+
+    if [ ! -f "$path_file" ]
+    then
+        echo "import sys; sys.__plen = len(sys.path)" >> "$path_file"
+        echo "import sys; new=sys.path[sys.__plen:]; del sys.path[sys.__plen:]; p=getattr(sys,'__egginsert',0); sys.path[p:p]=new; sys.__egginsert = p+len(new)" >> "$path_file"
+    fi
+
     for pydir in "$@"
     do
         absolute_path=$("$VIRTUALENVWRAPPER_PYTHON" -c "import os; print os.path.abspath(\"$pydir\")")
@@ -613,7 +640,16 @@ function add2virtualenv {
         then
             echo "Warning: Converting \"$pydir\" to \"$absolute_path\"" 1>&2
         fi
-        echo "$absolute_path" >> "$path_file"
+
+        if [ $remove -eq 1 ]
+        then
+            sed -i.tmp "\:^$absolute_path$: d" "$path_file"
+        else
+            sed -i.tmp '1 a\
+'$absolute_path'
+' "$path_file"
+        fi
+        rm -f "${path_file}.tmp"
     done
     return 0
 }
@@ -642,11 +678,11 @@ function lssitepackages {
     typeset site_packages="`virtualenvwrapper_get_site_packages_dir`"
     ls $@ $site_packages
     
-    path_file="$site_packages/virtualenv_path_extensions.pth"
+    path_file="$site_packages/_virtualenv_path_extensions.pth"
     if [ -f "$path_file" ]
     then
         echo
-        echo "virtualenv_path_extensions.pth:"
+        echo "_virtualenv_path_extensions.pth:"
         cat "$path_file"
     fi
 }
@@ -766,7 +802,7 @@ function mkproject_help {
     echo
     echo "Available project templates:"
     echo
-    "$VIRTUALENVWRAPPER_PYTHON" -m virtualenvwrapper.hook_loader -l project.template
+    "$VIRTUALENVWRAPPER_PYTHON" -c 'from virtualenvwrapper.hook_loader import main; main()' -l project.template
 }
 
 # Create a new project directory and its associated virtualenv.
@@ -842,7 +878,10 @@ function mkproject {
     do
         echo
         echo "Applying template $t"
-        virtualenvwrapper_run_hook --name $t project.template $envname
+        # For some reason zsh insists on prefixing the template
+        # names with a space, so strip them out before passing
+        # the value to the hook loader.
+        virtualenvwrapper_run_hook --name $(echo $t | sed 's/^ //') project.template $envname
     done
 
     virtualenvwrapper_run_hook project.post_mkproject
@@ -869,6 +908,51 @@ function cdproject {
     return 0
 }
 
+#
+# Temporary virtualenv
+#
+# Originally part of virtualenvwrapper.tmpenv plugin
+#
+mktmpenv() {
+    typeset tmpenvname
+
+    # Generate a unique temporary name, if one is not given.
+    if [ $# -eq 0 ]
+    then
+        tmpenvname=$("$VIRTUALENVWRAPPER_PYTHON" -c 'import uuid; print uuid.uuid4()' 2>/dev/null)
+        if [ -z "$tmpenvname" ]
+        then
+            # This python does not support uuid
+            tmpenvname=$("$VIRTUALENVWRAPPER_PYTHON" -c 'import random; print hex(random.getrandbits(64))[2:-1]' 2>/dev/null)
+        fi
+        mkvirtualenv "$tmpenvname"
+    else
+        mkvirtualenv "$@"
+    fi
+
+    # Create the environment
+    RC=$?
+    if [ $RC -ne 0 ]
+    then
+        return $RC
+    fi
+
+    # Change working directory
+    cdvirtualenv
+
+    # Create the tmpenv marker file
+    echo "This is a temporary environment. It will be deleted when you run 'deactivate'." | tee "$VIRTUAL_ENV/README.tmpenv"
+
+    # Update the postdeactivate script
+    cat - >> "$VIRTUAL_ENV/bin/postdeactivate" <<EOF
+if [ -f "$VIRTUAL_ENV/README.tmpenv" ]
+then
+    echo "Removing temporary environment:" $(basename "$VIRTUAL_ENV")
+    rmvirtualenv $(basename "$VIRTUAL_ENV")
+fi
+EOF
+}
+
 
 #
 # Invoke the initialization hooks
